package at.ac.uibk.gitsearch.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.ReviewComment;
import at.ac.uibk.gitsearch.domain.ReviewHistoryEntry;
import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import at.ac.uibk.gitsearch.repository.ReviewHistoryEntryRepository;
import at.ac.uibk.gitsearch.service.dto.ReviewHistoryEntryDTO;
import at.ac.uibk.gitsearch.service.mapper.ReviewHistoryEntryMapper;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link ReviewHistoryEntryResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
public class ReviewHistoryEntryResourceIT {

    private static final String DEFAULT_MESSAGE = "AAAAAAAAAA";
    private static final String UPDATED_MESSAGE = "BBBBBBBBBB";

    private static final ReviewStatus DEFAULT_TYPE = ReviewStatus.REVIEW_IN_PROGRESS;
    private static final ReviewStatus UPDATED_TYPE = ReviewStatus.IMPROVEMENTS_REQUESTED;

    private static final String ENTITY_API_URL = "/api/review-history-entries";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ReviewHistoryEntryRepository reviewHistoryEntryRepository;

    @Autowired
    private ReviewHistoryEntryMapper reviewHistoryEntryMapper;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restReviewHistoryEntryMockMvc;

    private ReviewHistoryEntry reviewHistoryEntry;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReviewHistoryEntry createEntity(EntityManager em) {
        ReviewHistoryEntry reviewHistoryEntry = new ReviewHistoryEntry().message(DEFAULT_MESSAGE).type(DEFAULT_TYPE);
        // Add required entity
        Review review;
        if (TestUtil.findAll(em, Review.class).isEmpty()) {
            review = ReviewResourceIT.createEntity(em);
            em.persist(review);
            em.flush();
        } else {
            review = TestUtil.findAll(em, Review.class).get(0);
        }
        reviewHistoryEntry.setReview(review);
        return reviewHistoryEntry;
    }

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public ReviewRating createReviewRatingEntity(EntityManager em) {
        Review review = ReviewResourceIT.createEntity(em);

        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();

        ReviewComment sampleComment = new ReviewComment();

        sampleComment.setDescriptionIsClear(false);
        sampleComment.setRequirementsAreClear(false);
        sampleComment.setDescriptionComment("");

        sampleComment.setInformationRequiredComplete(false);
        sampleComment.setAllMaterialsAvailable(false);
        sampleComment.setInformationRequiredComment("");

        sampleComment.setIsStructured(false);
        sampleComment.setSubExercisesAreClear(false);
        sampleComment.setStructuredComment("");

        sampleComment.setSolutionIsCorrect(false);
        sampleComment.setSolutionComment("");

        sampleComment.setMetadataIsComplete(false);
        sampleComment.setMetadataIsCorrect(false);
        sampleComment.setMetadataComment("");

        ReviewRating reviewRating = new ReviewRating();
        reviewRating.setStatus(DEFAULT_TYPE);
        reviewRating.setReview(review);
        reviewRating.setUser(user);
        reviewRating.setComment(sampleComment);

        return reviewRating;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReviewHistoryEntry createUpdatedEntity(EntityManager em) {
        ReviewHistoryEntry reviewHistoryEntry = new ReviewHistoryEntry().message(UPDATED_MESSAGE).type(UPDATED_TYPE);
        // Add required entity
        Review review;
        if (TestUtil.findAll(em, Review.class).isEmpty()) {
            review = ReviewResourceIT.createUpdatedEntity(em);
            em.persist(review);
            em.flush();
        } else {
            review = TestUtil.findAll(em, Review.class).get(0);
        }
        reviewHistoryEntry.setReview(review);
        return reviewHistoryEntry;
    }

    @BeforeEach
    public void initTest() {
        reviewHistoryEntry = createEntity(em);
    }

    @Test
    @Transactional
    void createReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeCreate = reviewHistoryEntryRepository.findAll().size();
        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);
        restReviewHistoryEntryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isCreated());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeCreate + 1);
        ReviewHistoryEntry testReviewHistoryEntry = reviewHistoryEntryList.get(reviewHistoryEntryList.size() - 1);
        assertThat(testReviewHistoryEntry.getMessage()).isEqualTo(DEFAULT_MESSAGE);
        assertThat(testReviewHistoryEntry.getType()).isEqualTo(DEFAULT_TYPE);
    }

    @Test
    @Transactional
    void createReviewHistoryEntryWithExistingId() throws Exception {
        // Create the ReviewHistoryEntry with an existing ID
        reviewHistoryEntry.setId(1L);
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        int databaseSizeBeforeCreate = reviewHistoryEntryRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restReviewHistoryEntryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkMessageIsRequired() throws Exception {
        int databaseSizeBeforeTest = reviewHistoryEntryRepository.findAll().size();
        // set the field null
        reviewHistoryEntry.setMessage(null);

        // Create the ReviewHistoryEntry, which fails.
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        restReviewHistoryEntryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkTypeIsRequired() throws Exception {
        int databaseSizeBeforeTest = reviewHistoryEntryRepository.findAll().size();
        // set the field null
        reviewHistoryEntry.setType(null);

        // Create the ReviewHistoryEntry, which fails.
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        restReviewHistoryEntryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntries() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(reviewHistoryEntry.getId().intValue())))
            .andExpect(jsonPath("$.[*].message").value(hasItem(DEFAULT_MESSAGE)))
            .andExpect(jsonPath("$.[*].type").value(hasItem(DEFAULT_TYPE.toString())));
    }

    @Test
    @Transactional
    void getReviewHistoryEntry() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get the reviewHistoryEntry
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL_ID, reviewHistoryEntry.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(reviewHistoryEntry.getId().intValue()))
            .andExpect(jsonPath("$.message").value(DEFAULT_MESSAGE))
            .andExpect(jsonPath("$.type").value(DEFAULT_TYPE.toString()));
    }

    @Test
    @Transactional
    void getReviewHistoryEntriesByIdFiltering() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        Long id = reviewHistoryEntry.getId();

        defaultReviewHistoryEntryShouldBeFound("id.equals=" + id);
        defaultReviewHistoryEntryShouldNotBeFound("id.notEquals=" + id);

        defaultReviewHistoryEntryShouldBeFound("id.greaterThanOrEqual=" + id);
        defaultReviewHistoryEntryShouldNotBeFound("id.greaterThan=" + id);

        defaultReviewHistoryEntryShouldBeFound("id.lessThanOrEqual=" + id);
        defaultReviewHistoryEntryShouldNotBeFound("id.lessThan=" + id);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message equals to DEFAULT_MESSAGE
        defaultReviewHistoryEntryShouldBeFound("message.equals=" + DEFAULT_MESSAGE);

        // Get all the reviewHistoryEntryList where message equals to UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldNotBeFound("message.equals=" + UPDATED_MESSAGE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageIsNotEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message not equals to DEFAULT_MESSAGE
        defaultReviewHistoryEntryShouldNotBeFound("message.notEquals=" + DEFAULT_MESSAGE);

        // Get all the reviewHistoryEntryList where message not equals to UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldBeFound("message.notEquals=" + UPDATED_MESSAGE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageIsInShouldWork() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message in DEFAULT_MESSAGE or UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldBeFound("message.in=" + DEFAULT_MESSAGE + "," + UPDATED_MESSAGE);

        // Get all the reviewHistoryEntryList where message equals to UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldNotBeFound("message.in=" + UPDATED_MESSAGE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageIsNullOrNotNull() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message is not null
        defaultReviewHistoryEntryShouldBeFound("message.specified=true");

        // Get all the reviewHistoryEntryList where message is null
        defaultReviewHistoryEntryShouldNotBeFound("message.specified=false");
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageContainsSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message contains DEFAULT_MESSAGE
        defaultReviewHistoryEntryShouldBeFound("message.contains=" + DEFAULT_MESSAGE);

        // Get all the reviewHistoryEntryList where message contains UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldNotBeFound("message.contains=" + UPDATED_MESSAGE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByMessageNotContainsSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where message does not contain DEFAULT_MESSAGE
        defaultReviewHistoryEntryShouldNotBeFound("message.doesNotContain=" + DEFAULT_MESSAGE);

        // Get all the reviewHistoryEntryList where message does not contain UPDATED_MESSAGE
        defaultReviewHistoryEntryShouldBeFound("message.doesNotContain=" + UPDATED_MESSAGE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByTypeIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where type equals to DEFAULT_TYPE
        defaultReviewHistoryEntryShouldBeFound("type.equals=" + DEFAULT_TYPE);

        // Get all the reviewHistoryEntryList where type equals to UPDATED_TYPE
        defaultReviewHistoryEntryShouldNotBeFound("type.equals=" + UPDATED_TYPE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByTypeIsNotEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where type not equals to DEFAULT_TYPE
        defaultReviewHistoryEntryShouldNotBeFound("type.notEquals=" + DEFAULT_TYPE);

        // Get all the reviewHistoryEntryList where type not equals to UPDATED_TYPE
        defaultReviewHistoryEntryShouldBeFound("type.notEquals=" + UPDATED_TYPE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByTypeIsInShouldWork() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where type in DEFAULT_TYPE or UPDATED_TYPE
        defaultReviewHistoryEntryShouldBeFound("type.in=" + DEFAULT_TYPE + "," + UPDATED_TYPE);

        // Get all the reviewHistoryEntryList where type equals to UPDATED_TYPE
        defaultReviewHistoryEntryShouldNotBeFound("type.in=" + UPDATED_TYPE);
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByTypeIsNullOrNotNull() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        // Get all the reviewHistoryEntryList where type is not null
        defaultReviewHistoryEntryShouldBeFound("type.specified=true");

        // Get all the reviewHistoryEntryList where type is null
        defaultReviewHistoryEntryShouldNotBeFound("type.specified=false");
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByReviewIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        Review review;
        if (TestUtil.findAll(em, Review.class).isEmpty()) {
            review = ReviewResourceIT.createEntity(em);
            em.persist(review);
            em.flush();
        } else {
            review = TestUtil.findAll(em, Review.class).get(0);
        }
        em.persist(review);
        em.flush();
        reviewHistoryEntry.setReview(review);
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        Long reviewId = review.getId();

        // Get all the reviewHistoryEntryList where review equals to reviewId
        defaultReviewHistoryEntryShouldBeFound("reviewId.equals=" + reviewId);

        // Get all the reviewHistoryEntryList where review equals to (reviewId + 1)
        defaultReviewHistoryEntryShouldNotBeFound("reviewId.equals=" + (reviewId + 1));
    }

    @Test
    @Transactional
    void getAllReviewHistoryEntriesByReviewRatingIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        ReviewRating reviewRating;
        if (TestUtil.findAll(em, ReviewRating.class).isEmpty()) {
            reviewRating = createReviewRatingEntity(em);
            em.persist(reviewRating);
            em.flush();
        } else {
            reviewRating = TestUtil.findAll(em, ReviewRating.class).get(0);
        }
        em.persist(reviewRating);
        em.flush();
        reviewHistoryEntry.setReviewRating(reviewRating);
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        Long reviewRatingId = reviewRating.getId();

        // Get all the reviewHistoryEntryList where reviewRating equals to reviewRatingId
        defaultReviewHistoryEntryShouldBeFound("reviewRatingId.equals=" + reviewRatingId);

        // Get all the reviewHistoryEntryList where reviewRating equals to (reviewRatingId + 1)
        defaultReviewHistoryEntryShouldNotBeFound("reviewRatingId.equals=" + (reviewRatingId + 1));
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void getAllReviewHistoryEntriesByUserIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        User user;
        if (TestUtil.findAll(em, User.class).isEmpty()) {
            user = UserResourceIT.createEntity(em);
            em.persist(user);
            em.flush();
        } else {
            user = TestUtil.findAll(em, User.class).get(0);
        }
        em.persist(user);
        em.flush();
        reviewHistoryEntry.setUser(user);
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);
        Long userId = user.getId();

        // Get all the reviewHistoryEntryList where user equals to userId
        defaultReviewHistoryEntryShouldBeFound("userId.equals=" + userId);

        // Get all the reviewHistoryEntryList where user equals to (userId + 1)
        defaultReviewHistoryEntryShouldNotBeFound("userId.equals=" + (userId + 1));
    }

    /**
     * Executes the search, and checks that the default entity is returned.
     */
    private void defaultReviewHistoryEntryShouldBeFound(String filter) throws Exception {
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc&" + filter))
            .andDo(print())
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(reviewHistoryEntry.getId().intValue())))
            .andExpect(jsonPath("$.[*].message").value(hasItem(DEFAULT_MESSAGE)))
            .andExpect(jsonPath("$.[*].type").value(hasItem(DEFAULT_TYPE.toString())));

        // Check, that the count call also returns 1
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL + "/count?sort=id,desc&" + filter).with(csrf().asHeader()))
            .andDo(print())
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("1"));
    }

    /**
     * Executes the search, and checks that the default entity is not returned.
     */
    private void defaultReviewHistoryEntryShouldNotBeFound(String filter) throws Exception {
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc&" + filter))
            .andDo(print())
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$").isArray())
            .andExpect(jsonPath("$").isEmpty());

        // Check, that the count call also returns 0
        restReviewHistoryEntryMockMvc
            .perform(get(ENTITY_API_URL + "/count?sort=id,desc&" + filter))
            .andDo(print())
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("0"));
    }

    @Test
    @Transactional
    void getNonExistingReviewHistoryEntry() throws Exception {
        // Get the reviewHistoryEntry
        restReviewHistoryEntryMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewReviewHistoryEntry() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();

        // Update the reviewHistoryEntry
        ReviewHistoryEntry updatedReviewHistoryEntry = reviewHistoryEntryRepository.findById(reviewHistoryEntry.getId()).get();
        // Disconnect from session so that the updates on updatedReviewHistoryEntry are not directly saved in db
        em.detach(updatedReviewHistoryEntry);
        updatedReviewHistoryEntry.message(UPDATED_MESSAGE).type(UPDATED_TYPE);
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(updatedReviewHistoryEntry);

        restReviewHistoryEntryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, reviewHistoryEntryDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
        ReviewHistoryEntry testReviewHistoryEntry = reviewHistoryEntryList.get(reviewHistoryEntryList.size() - 1);
        assertThat(testReviewHistoryEntry.getMessage()).isEqualTo(UPDATED_MESSAGE);
        assertThat(testReviewHistoryEntry.getType()).isEqualTo(UPDATED_TYPE);
    }

    @Test
    @Transactional
    void putNonExistingReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, reviewHistoryEntryDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateReviewHistoryEntryWithPatch() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();

        // Update the reviewHistoryEntry using partial update
        ReviewHistoryEntry partialUpdatedReviewHistoryEntry = new ReviewHistoryEntry();
        partialUpdatedReviewHistoryEntry.setId(reviewHistoryEntry.getId());

        partialUpdatedReviewHistoryEntry.message(UPDATED_MESSAGE).type(UPDATED_TYPE);

        restReviewHistoryEntryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReviewHistoryEntry.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedReviewHistoryEntry))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
        ReviewHistoryEntry testReviewHistoryEntry = reviewHistoryEntryList.get(reviewHistoryEntryList.size() - 1);
        assertThat(testReviewHistoryEntry.getMessage()).isEqualTo(UPDATED_MESSAGE);
        assertThat(testReviewHistoryEntry.getType()).isEqualTo(UPDATED_TYPE);
    }

    @Test
    @Transactional
    void fullUpdateReviewHistoryEntryWithPatch() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();

        // Update the reviewHistoryEntry using partial update
        ReviewHistoryEntry partialUpdatedReviewHistoryEntry = new ReviewHistoryEntry();
        partialUpdatedReviewHistoryEntry.setId(reviewHistoryEntry.getId());

        partialUpdatedReviewHistoryEntry.message(UPDATED_MESSAGE).type(UPDATED_TYPE);

        restReviewHistoryEntryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReviewHistoryEntry.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedReviewHistoryEntry))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
        ReviewHistoryEntry testReviewHistoryEntry = reviewHistoryEntryList.get(reviewHistoryEntryList.size() - 1);
        assertThat(testReviewHistoryEntry.getMessage()).isEqualTo(UPDATED_MESSAGE);
        assertThat(testReviewHistoryEntry.getType()).isEqualTo(UPDATED_TYPE);
    }

    @Test
    @Transactional
    void patchNonExistingReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, reviewHistoryEntryDTO.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamReviewHistoryEntry() throws Exception {
        int databaseSizeBeforeUpdate = reviewHistoryEntryRepository.findAll().size();
        reviewHistoryEntry.setId(count.incrementAndGet());

        // Create the ReviewHistoryEntry
        ReviewHistoryEntryDTO reviewHistoryEntryDTO = reviewHistoryEntryMapper.toDto(reviewHistoryEntry);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewHistoryEntryMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewHistoryEntryDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReviewHistoryEntry in the database
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteReviewHistoryEntry() throws Exception {
        // Initialize the database
        reviewHistoryEntryRepository.saveAndFlush(reviewHistoryEntry);

        int databaseSizeBeforeDelete = reviewHistoryEntryRepository.findAll().size();

        // Delete the reviewHistoryEntry
        restReviewHistoryEntryMockMvc
            .perform(delete(ENTITY_API_URL_ID, reviewHistoryEntry.getId()).accept(MediaType.APPLICATION_JSON).with(csrf().asHeader()))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<ReviewHistoryEntry> reviewHistoryEntryList = reviewHistoryEntryRepository.findAll();
        assertThat(reviewHistoryEntryList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
